@@ -31,6 +31,7 @@ modules = [
     "batoto",
     "bbc",
     "behance",
+    "bellazon",
     "bilibili",
     "blogger",
     "bluesky",
@@ -44,6 +45,7 @@ modules = [
     "comick",
     "comicvine",
     "cyberdrop",
+    "cyberfile",
     "danbooru",
     "dankefuerslesen",
     "desktopography",
@@ -170,6 +172,7 @@ modules = [
     "senmanga",
     "sexcom",
     "shimmie2",
+    "simpcity",
     "simplyhentai",
     "sizebooru",
     "skeb",
